@@ -225,9 +225,18 @@ function genericPrintNoParens(path, options, print) {
         return path.call(print, "program");
 
     case "Program":
-        return path.call(function(bodyPath) {
+        // Babel 6
+        if (n.directives) {
+            path.each(function(childPath) {
+                parts.push(print(childPath), ";\n");
+            }, "directives");
+        }
+
+        parts.push(path.call(function(bodyPath) {
             return printStatementSequence(bodyPath, options, print);
-        }, "body");
+        }, "body"));
+
+        return concat(parts);
 
     case "Noop": // Babel extension.
     case "EmptyStatement":
@@ -251,7 +260,7 @@ function genericPrintNoParens(path, options, print) {
     case "AssignmentPattern":
         return concat([
             path.call(print, "left"),
-            "=",
+            " = ",
             path.call(print, "right")
         ]);
 
@@ -333,6 +342,10 @@ function genericPrintNoParens(path, options, print) {
         if (n.async)
             parts.push("async ");
 
+        if (n.typeParameters) {
+            parts.push(path.call(print, "typeParameters"));
+        }
+
         if (
             !options.arrowParensAlways &&
             n.params.length === 1 &&
@@ -355,7 +368,6 @@ function genericPrintNoParens(path, options, print) {
 
         return concat(parts);
 
-    case "ClassMethod": // Babel 6
     case "MethodDefinition":
         if (n.static) {
             parts.push("static ");
@@ -501,7 +513,9 @@ function genericPrintNoParens(path, options, print) {
                     namedTypes.ImportSpecifier.assert(value);
                     if (!foundImportSpecifier) {
                         foundImportSpecifier = true;
-                        parts.push("{");
+                        parts.push(
+                          options.objectCurlySpacing ? "{ " : "{"
+                        );
                     }
                 }
 
@@ -509,7 +523,9 @@ function genericPrintNoParens(path, options, print) {
             }, "specifiers");
 
             if (foundImportSpecifier) {
-                parts.push("}");
+                parts.push(
+                  options.objectCurlySpacing ? " }" : "}"
+                );
             }
 
             parts.push(" from ");
@@ -524,15 +540,28 @@ function genericPrintNoParens(path, options, print) {
             return printStatementSequence(bodyPath, options, print);
         }, "body");
 
+
         if (naked.isEmpty()) {
-            return fromString("{}");
+            if (!n.directives || n.directives.length === 0) {
+                return fromString("{}");
+            }
         }
 
-        return concat([
-            "{\n",
-            naked.indent(options.tabWidth),
-            "\n}"
-        ]);
+        parts.push("{\n");
+        // Babel 6
+        if (n.directives) {
+            path.each(function(childPath) {
+                parts.push(
+                    print(childPath).indent(options.tabWidth),
+                    ";",
+                    n.directives.length > 1 || !naked.isEmpty() ? "\n" : ""
+                );
+            }, "directives");
+        }
+        parts.push(naked.indent(options.tabWidth));
+        parts.push("\n}");
+
+        return concat(parts);
 
     case "ReturnStatement":
         parts.push("return");
@@ -582,7 +611,10 @@ function genericPrintNoParens(path, options, print) {
         });
 
         var oneLine = (isTypeAnnotation && len === 1) || len === 0;
-        parts.push(oneLine ? "{" : "{\n");
+        var leftBrace = n.exact ? "{|" : "{";
+        var rightBrace = n.exact ? "|}" : "}";
+        parts.push(oneLine ? leftBrace : leftBrace + "\n");
+        var leftBraceIndex = parts.length - 1;
 
         var i = 0;
         fields.forEach(function(field) {
@@ -608,14 +640,19 @@ function genericPrintNoParens(path, options, print) {
                     allowBreak = !multiLine;
                 } else if (len !== 1 && isTypeAnnotation) {
                     parts.push(separator);
-                } else if (!oneLine && options.trailingComma) {
+                } else if (!oneLine && util.isTrailingCommaEnabled(options, "objects")) {
                     parts.push(separator);
                 }
                 i++;
             }, field);
         });
 
-        parts.push(oneLine ? "}" : "\n}");
+        parts.push(oneLine ? rightBrace : "\n" + rightBrace);
+
+        if (i !== 0 && oneLine && options.objectCurlySpacing) {
+            parts[leftBraceIndex] = leftBrace + " ";
+            parts[parts.length - 1] = " " + rightBrace;
+        }
 
         return concat(parts);
 
@@ -627,7 +664,6 @@ function genericPrintNoParens(path, options, print) {
         ]);
 
     case "ObjectProperty": // Babel 6
-    case "ObjectMethod": // Babel 6
     case "Property": // Non-standard AST node type.
         if (n.method || n.kind === "get" || n.kind === "set") {
             return printMethod(path, options, print);
@@ -646,6 +682,16 @@ function genericPrintNoParens(path, options, print) {
 
         return concat(parts);
 
+    case "ClassMethod": // Babel 6
+        if (n.static) {
+            parts.push("static ");
+        }
+
+        return concat([parts, printObjectMethod(path, options, print)]);
+
+    case "ObjectMethod": // Babel 6
+        return printObjectMethod(path, options, print);
+
     case "Decorator":
         return concat(["@", path.call(print, "expression")]);
 
@@ -657,7 +703,15 @@ function genericPrintNoParens(path, options, print) {
         var printed = path.map(print, "elements");
         var joined = fromString(", ").join(printed);
         var oneLine = joined.getLineLength(1) <= options.wrapColumn;
-        parts.push(oneLine ? "[" : "[\n");
+        if (oneLine) {
+          if (options.arrayBracketSpacing) {
+            parts.push("[ ");
+          } else {
+            parts.push("[");
+          }
+        } else {
+          parts.push("[\n");
+        }
 
         path.each(function(elemPath) {
             var i = elemPath.getName();
@@ -678,14 +732,18 @@ function genericPrintNoParens(path, options, print) {
                     lines = lines.indent(options.tabWidth);
                 }
                 parts.push(lines);
-                if (i < len - 1 || (!oneLine && options.trailingComma))
+                if (i < len - 1 || (!oneLine && util.isTrailingCommaEnabled(options, "arrays")))
                     parts.push(",");
                 if (!oneLine)
                     parts.push("\n");
             }
         }, "elements");
 
-        parts.push("]");
+        if (oneLine && options.arrayBracketSpacing) {
+          parts.push(" ]");
+        } else {
+          parts.push("]");
+        }
 
         return concat(parts);
 
@@ -698,17 +756,27 @@ function genericPrintNoParens(path, options, print) {
     case "Super":
         return fromString("super");
 
-    case "BooleanLiteral": // Babel 6 Literal split
+    case "NullLiteral": // Babel 6 Literal split
+        return fromString("null");
+
     case "RegExpLiteral": // Babel 6 Literal split
+        return fromString(n.extra.raw);
+
+    case "BooleanLiteral": // Babel 6 Literal split
     case "NumericLiteral": // Babel 6 Literal split
     case "StringLiteral": // Babel 6 Literal split
-    case "NullLiteral": // Babel 6 Literal split
     case "Literal":
         if (typeof n.value !== "string")
             return fromString(n.value, options);
 
         return fromString(nodeStr(n.value, options), options);
 
+    case "Directive": // Babel 6
+        return path.call(print, "value");
+
+    case "DirectiveLiteral": // Babel 6
+        return fromString(nodeStr(n.value, options));
+
     case "ModuleSpecifier":
         if (n.local) {
             throw new Error(
@@ -780,7 +848,9 @@ function genericPrintNoParens(path, options, print) {
         if (!namedTypes.ForStatement.check(parentNode) &&
             !namedTypes.ForInStatement.check(parentNode) &&
             !(namedTypes.ForOfStatement &&
-              namedTypes.ForOfStatement.check(parentNode))) {
+              namedTypes.ForOfStatement.check(parentNode)) &&
+            !(namedTypes.ForAwaitStatement &&
+              namedTypes.ForAwaitStatement.check(parentNode))) {
             parts.push(";");
         }
 
@@ -863,6 +933,16 @@ function genericPrintNoParens(path, options, print) {
             adjustClause(path.call(print, "body"), options)
         ]);
 
+    case "ForAwaitStatement":
+        return concat([
+            "for await (",
+            path.call(print, "left"),
+            " of ",
+            path.call(print, "right"),
+            ")",
+            adjustClause(path.call(print, "body"), options)
+        ]);
+
     case "DoWhileStatement":
         var doBody = concat([
             "do",
@@ -1103,7 +1183,16 @@ function genericPrintNoParens(path, options, print) {
         if (n.static)
             parts.push("static ");
 
-        parts.push(path.call(print, "key"));
+        var key = path.call(print, "key");
+        if (n.computed) {
+            key = concat(["[", key, "]"]);
+        } else if (n.variance === "plus") {
+            key = concat(["+", key]);
+        } else if (n.variance === "minus") {
+            key = concat(["-", key]);
+        }
+        parts.push(key);
+
         if (n.typeAnnotation)
             parts.push(path.call(print, "typeAnnotation"));
 
@@ -1209,9 +1298,13 @@ function genericPrintNoParens(path, options, print) {
 
         return fromString("");
 
+    case "ExistentialTypeParam":
     case "ExistsTypeAnnotation":
         return fromString("*", options);
 
+    case "EmptyTypeAnnotation":
+        return fromString("empty", options);
+
     case "AnyTypeAnnotation":
         return fromString("any", options);
 
@@ -1254,6 +1347,12 @@ function genericPrintNoParens(path, options, print) {
             path.call(print, "body"),
         ]);
 
+    case "DeclareModuleExports":
+        return printFlowDeclaration(path, [
+            "module.exports",
+            path.call(print, "typeAnnotation"),
+        ]);
+
     case "DeclareVariable":
         return printFlowDeclaration(path, [
             "var ",
@@ -1367,7 +1466,12 @@ function genericPrintNoParens(path, options, print) {
         return path.call(print, "value");
 
     case "ObjectTypeIndexer":
+        var variance =
+            n.variance === "plus" ? "+" :
+            n.variance === "minus" ? "-" : "";
+
         return concat([
+            variance,
             "[",
             path.call(print, "id"),
             ": ",
@@ -1377,7 +1481,12 @@ function genericPrintNoParens(path, options, print) {
         ]);
 
     case "ObjectTypeProperty":
+        var variance =
+            n.variance === "plus" ? "+" :
+            n.variance === "minus" ? "-" : "";
+
         return concat([
+            variance,
             path.call(print, "key"),
             n.optional ? "?" : "",
             ": ",
@@ -1408,6 +1517,7 @@ function genericPrintNoParens(path, options, print) {
         return concat([
             "type ",
             path.call(print, "id"),
+            path.call(print, "typeParameters"),
             " = ",
             path.call(print, "right"),
             ";"
@@ -1649,7 +1759,11 @@ function printMethod(path, options, print) {
     var kind = node.kind;
     var parts = [];
 
-    namedTypes.FunctionExpression.assert(node.value);
+    if (node.type === "ObjectMethod" || node.type === "ClassMethod") {
+        node.value = node;
+    } else {
+        namedTypes.FunctionExpression.assert(node.value);
+    }
 
     if (node.value.async) {
         parts.push("async ");
@@ -1687,6 +1801,7 @@ function printMethod(path, options, print) {
 
 function printArgumentsList(path, options, print) {
     var printed = path.map(print, "arguments");
+    var trailingComma = util.isTrailingCommaEnabled(options, "parameters");
 
     var joined = fromString(", ").join(printed);
     if (joined.getLineLength(1) > options.wrapColumn) {
@@ -1694,7 +1809,7 @@ function printArgumentsList(path, options, print) {
         return concat([
             "(\n",
             joined.indent(options.tabWidth),
-            options.trailingComma ? ",\n)" : "\n)"
+            trailingComma ? ",\n)" : "\n)"
         ]);
     }
 
@@ -1703,6 +1818,7 @@ function printArgumentsList(path, options, print) {
 
 function printFunctionParams(path, options, print) {
     var fun = path.getValue();
+
     namedTypes.Function.assert(fun);
 
     var printed = path.map(print, "params");
@@ -1712,7 +1828,7 @@ function printFunctionParams(path, options, print) {
             var i = defExprPath.getName();
             var p = printed[i];
             if (p && defExprPath.getValue()) {
-                printed[i] = concat([p, "=", print(defExprPath)]);
+                printed[i] = concat([p, " = ", print(defExprPath)]);
             }
         }, "defaults");
     }
@@ -1725,8 +1841,12 @@ function printFunctionParams(path, options, print) {
     if (joined.length > 1 ||
         joined.getLineLength(1) > options.wrapColumn) {
         joined = fromString(",\n").join(printed);
-        if (options.trailingComma && !fun.rest) {
+        if (util.isTrailingCommaEnabled(options, "parameters") &&
+            !fun.rest &&
+            fun.params[fun.params.length - 1].type !== 'RestElement') {
             joined = concat([joined, ",\n"]);
+        } else {
+            joined = concat([joined, "\n"]);
         }
         return concat(["\n", joined.indent(options.tabWidth)]);
     }
@@ -1734,9 +1854,43 @@ function printFunctionParams(path, options, print) {
     return joined;
 }
 
+function printObjectMethod(path, options, print) {
+    var objMethod = path.getValue();
+    var parts = [];
+
+    if (objMethod.async)
+        parts.push("async ");
+
+    if (objMethod.generator)
+        parts.push("*");
+
+    if (objMethod.method || objMethod.kind === "get" || objMethod.kind === "set") {
+        return printMethod(path, options, print);
+    }
+
+    var key = path.call(print, "key");
+    if (objMethod.computed) {
+        parts.push("[", key, "]");
+    } else {
+        parts.push(key);
+    }
+
+    parts.push(
+        "(",
+        printFunctionParams(path, options, print),
+        ")",
+        path.call(print, "returnType"),
+        " ",
+        path.call(print, "body")
+    );
+
+    return concat(parts);
+}
+
 function printExportDeclaration(path, options, print) {
     var decl = path.getValue();
     var parts = ["export "];
+    var shouldPrintSpaces = options.objectCurlySpacing;
 
     namedTypes.Declaration.assert(decl);
 
@@ -1756,9 +1910,9 @@ function printExportDeclaration(path, options, print) {
             parts.push("*");
         } else {
             parts.push(
-                "{",
+                shouldPrintSpaces ? "{ " : "{",
                 fromString(", ").join(path.map(print, "specifiers")),
-                "}"
+                shouldPrintSpaces ? " }" : "}"
             );
         }
 
