@@ -280,7 +280,8 @@ describe("printer", function() {
         "              param */ a,",
         "  // other params",
         "  b, c, // see?",
-        "  d) {",
+        "  d",
+        ") {",
         "  return a + b + c + d;",
         "}"
     ];
@@ -294,7 +295,8 @@ describe("printer", function() {
         "  b,",
         "  // see?",
         "  c,",
-        "  d) {",
+        "  d",
+        ") {",
         "  return a + b + c + d;",
         "}"
     ];
@@ -487,7 +489,7 @@ describe("printer", function() {
 
         assert.strictEqual(
             printer.print(funExpr).code,
-            "function a(b, c=1, ...d) {}"
+            "function a(b, c = 1, ...d) {}"
         );
 
         var arrowFunExpr = b.arrowFunctionExpression(
@@ -503,7 +505,7 @@ describe("printer", function() {
 
         assert.strictEqual(
             printer.print(arrowFunExpr).code,
-            "(b, c=1, ...d) => {}"
+            "(b, c = 1, ...d) => {}"
         );
     });
 
@@ -813,6 +815,15 @@ describe("printer", function() {
 
         var pretty = printer.printGenerically(ast).code;
         assert.strictEqual(pretty, code);
+
+        // It should also work when using the `trailingComma` option as an object.
+        printer = new Printer({
+          tabWidth: 2,
+          trailingComma: { objects: true },
+        });
+
+        pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
     });
 
     it("prints trailing commas in function calls", function() {
@@ -836,6 +847,16 @@ describe("printer", function() {
 
         var pretty = printer.printGenerically(ast).code;
         assert.strictEqual(pretty, code);
+
+        // It should also work when using the `trailingComma` option as an object.
+        printer = new Printer({
+          tabWidth: 2,
+          wrapColumn: 1,
+          trailingComma: { parameters: true },
+        });
+
+        pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
     });
 
     it("prints trailing commas in array expressions", function() {
@@ -859,6 +880,16 @@ describe("printer", function() {
 
         var pretty = printer.printGenerically(ast).code;
         assert.strictEqual(pretty, code);
+
+        // It should also work when using the `trailingComma` option as an object.
+        printer = new Printer({
+          tabWidth: 2,
+          wrapColumn: 1,
+          trailingComma: { arrays: true },
+        });
+
+        pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
     });
 
     it("prints trailing commas in function definitions", function() {
@@ -882,11 +913,45 @@ describe("printer", function() {
 
         var pretty = printer.printGenerically(ast).code;
         assert.strictEqual(pretty, code);
+
+        // It should also work when using the `trailingComma` option as an object.
+        printer = new Printer({
+          tabWidth: 2,
+          wrapColumn: 1,
+          trailingComma: { parameters: true },
+        });
+
+        pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
+    });
+
+    it("shouldn't print a trailing comma for a RestElement", function() {
+        var code = [
+            "function foo(",
+            "  a,",
+            "  b,",
+            "  ...rest",
+            ") {}"
+        ].join(eol);
+
+        var ast = parse(code, {
+            // The flow parser and Babylon recognize `...rest` as a `RestElement`
+            parser: require("babylon")
+        });
+
+        var printer = new Printer({
+            tabWidth: 2,
+            wrapColumn: 1,
+            trailingComma: true,
+        });
+
+        var pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
     });
 
     it("should support AssignmentPattern and RestElement", function() {
         var code = [
-            "function foo(a, [b, c]=d(a), ...[e, f, ...rest]) {",
+            "function foo(a, [b, c] = d(a), ...[e, f, ...rest]) {",
             "  return [a, b, c, e, f, rest];",
             "}"
         ].join(eol);
@@ -1384,8 +1449,8 @@ describe("printer", function() {
     });
 
     it("shouldn't print a trailing comma for single-line flow object types", function() {
-        var code1 = "type MyType = {message: string};";
-        var code2 = "type MyType = {[key: string]: string};";
+        var code1 = "type MyType = { message: string };";
+        var code2 = "type MyType = { [key: string]: string };";
 
         var ast1 = b.typeAlias(
             b.identifier("MyType"),
@@ -1466,4 +1531,78 @@ describe("printer", function() {
         var pretty = printer.printGenerically(ast).code;
         assert.strictEqual(pretty, code);
     });
+
+    it("uses the `arrayBracketSpacing` and the `objectCurlySpacing` option", function() {
+      var babylon = require("babylon");
+      var parseOptions = {
+        parser: {
+          parse: function (source) {
+            return babylon.parse(source, {
+              sourceType: 'module',
+              plugins: ['flow'],
+            });
+          }
+        }
+      };
+
+      var testCaseList = [{
+        printerConfig: {arrayBracketSpacing: false, objectCurlySpacing: false},
+        code: [
+          'import {java, script} from "javascript";',
+          '',
+          'function foo(a) {',
+          '    type MyType = {message: string};',
+          '    return [1, 2, 3];',
+          '}',
+          '',
+          'export {foo};'
+        ].join(eol)
+      }, {
+        printerConfig: {arrayBracketSpacing: true, objectCurlySpacing: false},
+        code: [
+          'import {java, script} from "javascript";',
+          '',
+          'function foo(a) {',
+          '    type MyType = {message: string};',
+          '    return [ 1, 2, 3 ];',
+          '}',
+          '',
+          'export {foo};'
+        ].join(eol)
+      }, {
+        printerConfig: {arrayBracketSpacing: false, objectCurlySpacing: true},
+        code: [
+          'import { java, script } from "javascript";',
+          '',
+          'function foo(a) {',
+          '    type MyType = { message: string };',
+          '    return [1, 2, 3];',
+          '}',
+          '',
+          'export { foo };'
+        ].join(eol)
+      }, {
+        printerConfig: {arrayBracketSpacing: true, objectCurlySpacing: true},
+        code: [
+          'import { java, script } from "javascript";',
+          '',
+          'function foo(a) {',
+          '    type MyType = { message: string };',
+          '    return [ 1, 2, 3 ];',
+          '}',
+          '',
+          'export { foo };'
+        ].join(eol)
+      }];
+
+      testCaseList.forEach(function(testCase) {
+        var code = testCase.code;
+        var printer = new Printer(testCase.printerConfig);
+
+        var ast = parse(code, parseOptions);
+        var pretty = printer.printGenerically(ast).code;
+
+        assert.strictEqual(pretty, code);
+      });
+    });
 });
